require 'spec_helper'

describe 'puppet_enterprise::master::puppetserver' do
  before :all do
    @facter_facts = {
      'osfamily'          => 'Debian',
      'operatingsystem'   => 'Debian',
      'lsbmajdistrelease' => '6',
      'puppetversion'     => '3.6.2 (Puppet Enterprise 3.3.0)',
      'is_pe'             => 'true',
      'fqdn'              => 'master.rspec',
      'clientcert'        => 'awesomecert',
      'pe_concat_basedir' => '/tmp/file',
      'processorcount'    => '1',
    }

    @params = {
      'certname'                                 => 'master.rspec',
      'metrics_enabled'                          => true,
      'metrics_server_id'                        => 'localhost',
      'metrics_jmx_enabled'                      => true,
      'metrics_graphite_enabled'                 => false,
      'metrics_graphite_host'                    => 'graphite',
      'metrics_graphite_port'                    => 2003,
      'metrics_graphite_update_interval_seconds' => 30,
      'profiler_enabled'                         => true,
      'ssl_protocols'                            => ['TLSv1', 'TLSv1.1', 'TLSv1.2'],
      'cipher_suites'                            => ['TLS_RSA_WITH_AES_256_CBC_SHA256', 
	                                             'TLS_RSA_WITH_AES_256_CBC_SHA', 
                                                     'TLS_RSA_WITH_AES_128_CBC_SHA256', 
                                                     'TLS_RSA_WITH_AES_128_CBC_SHA'],
      'idle_timeout_milliseconds'                => 12000000,
      'connect_timeout_milliseconds'             => 1200000,
    }
  end

  let(:facts) { @facter_facts }
  let(:params) { @params }
  let(:confdir) { "/etc/puppetlabs/puppetserver" }

  context 'puppet-admin certs' do
    context 'default list' do
      it { should contain_file("#{confdir}/conf.d/pe-puppet-server.conf").with_content(%r[puppet-admin:.*\n.*client-whitelist:\s*\[\s*pe-internal-classifier\s*\]]) }
    end

    context 'using custom certs' do
      before :each do
        @params['puppet_admin_certs'] = ['foo', 'bar']
      end

      it { should contain_file("#{confdir}/conf.d/pe-puppet-server.conf").with_content(%r[puppet-admin:.*\n.*client-whitelist:\s*\[\s*pe-internal-classifier, foo, bar]) }
    end
  end

  context "managing files" do
    it { should contain_pe_concat("#{confdir}/bootstrap.cfg").with_mode('0640') }
    it { should contain_file("#{confdir}/conf.d/webserver.conf").with_mode('0640') }
    it { should contain_file("#{confdir}/conf.d/webserver.conf").with_content(%r[ssl-ca-cert\s*:\s*/etc/puppetlabs/puppet/ssl/certs/ca.pem]) }
    it { should contain_file("#{confdir}/conf.d/webserver.conf").with_content(%r[ssl-crl-path\s*:\s*/etc/puppetlabs/puppet/ssl/crl.pem]) }
    it { should contain_file("#{confdir}/conf.d/webserver.conf").with_content(%r[access-log-config\s*:.*request-logging\.xml]) }
    it { should contain_file("#{confdir}/conf.d/global.conf").with_mode('0640') }
    it { should contain_file("#{confdir}/conf.d/pe-puppet-server.conf").with_mode('0640') }
    it { should contain_file("#{confdir}/conf.d/metrics.conf").with_mode('0640') }
    it { should_not contain_file("#{confdir}/conf.d/webserver.conf").with_content(%r[max-threads:\s*\d*]) }
    it { should_not contain_file("#{confdir}/conf.d/pe-puppet-server.conf").with_content(%r[max-active-instances\s*:\s*5]) }
    it { should contain_file("#{confdir}/conf.d/pe-puppet-server.conf").with_content(%r[ssl-protocols\s*:\s*\[\s*TLSv1, TLSv1.1, TLSv1.2\s*\]]) }
    it { should contain_file("#{confdir}/conf.d/pe-puppet-server.conf").with_content(%r[cipher-suites\s*:\s*\[\s*TLS_RSA_WITH_AES_256_CBC_SHA256, TLS_RSA_WITH_AES_256_CBC_SHA, TLS_RSA_WITH_AES_128_CBC_SHA256, TLS_RSA_WITH_AES_128_CBC_SHA\s*\]]) }
    it { should contain_file("#{confdir}/conf.d/pe-puppet-server.conf").with_content(%r[idle-timeout-milliseconds\s*:\s*1200000\s*]) }
    it { should contain_file("#{confdir}/conf.d/pe-puppet-server.conf").with_content(%r[connect-timeout-milliseconds\s*:\s*120000\s*]) }
  end

  context "managing jetty max-threads" do
    before :each do
      @params['tk_jetty_max_threads'] = '10'
    end

    it { should contain_file("#{confdir}/conf.d/webserver.conf").with_content(%r[max-threads\s*:\s*10]) }
  end

  context "managing the number of jruby instances" do
    before :each do
      @params['jruby_max_active_instances'] = '5'
    end

    it { should contain_file("#{confdir}/conf.d/pe-puppet-server.conf").with_content(%r[max-active-instances\s*:\s*5]) }
  end

  context "serving static files" do
    before :each do
      @params['static_files'] = {"/baz-files" => "/foo/bar/baz",
                                 "/others" => "/foo/bar/others"}
    end
    it { should contain_file("#{confdir}/conf.d/webserver.conf").with_content %r[resource: "/foo/bar/baz", path: "/baz-files", follow-links: true] }
    it { should contain_file("#{confdir}/conf.d/webserver.conf").with_content %r[resource: "/foo/bar/others", path: "/others", follow-links: true] }
  end

  context "managing services" do
    it { should contain_service('pe-puppetserver').with_ensure('running') }
  end
end
